def init_cond(Ts,riser,wtg,gt,zstep,tvd,deltaz):
    # Initial Conditions
    Tdsio = [Ts]   # Temperature of the fluid inside the drill string at RKB
    Tdso = [Ts]    # Temperature of the drill string wall at RKB, t=0
    Tao = [Ts]      # Temperature of the fluid inside the annulus at RKB, t=0
    Tcsgo = [Ts]    # Temperature of the casing at RKB, t=0   
    Tsro = [Ts]    # Temperature of the surrounding space at RKB, t=0
    Tfm = [Ts]      # Temperature of the formation at RKB

    for j in range(1, zstep):

        if j <= riser:
            Tg = wtg    # Water Thermal Gradient for the Riser section
        else:
            Tg = gt      # Geothermal Gradient below the Riser section

        deltaT = Tsro[j - 1] + Tg*(tvd[j]-tvd[j-1])/deltaz
        # Generating the Temperature Profile at t=0
        Tdsio.append(deltaT)
        Tdso.append(deltaT)
        Tao.append(deltaT)
        Tcsgo.append(deltaT)
        Tsro.append(deltaT)
        Tfm.append(deltaT)
    return Tdsio, Tdso, Tao, Tcsgo, Tsro, Tfm



from statistics import mean


def temp_time(n):

    # Simulation main parameters
    time = n  # circulating time, h
    tcirc = time * 3600  # circulating time, s
    tstep = 1
    deltat = tcirc / tstep

    return tcirc,tstep,deltat


def stab_time():
    Ta = []
    for n in range(1,3):
        Ta.append(temp_time(n)[1])

    valor = mean(Ta[0]) - mean(Ta[1])
    finaltime = 2

    while abs(valor) >= 0.01:
        Ta.append(temp_time(finaltime+1)[1])
        valor = mean(Ta[finaltime]) - mean(Ta[finaltime-1])
        finaltime = finaltime+1

    Tbot = []
    Tout = []

    for n in range(finaltime):
        Tbot.append(Ta[[n][-1]])
        Tout.append(Ta[[n][0]])

    return finaltime, Tbot, Tout





def wellpath(mdt):
    #WELLPATH
    md=range(mdt)  # Measured Depth from RKB, m
    tvd=[]   # True Vertical Depth from RKB, m

    for z in md:
        tvd.append(z)

    deltaz = 50  # Length of each grid cell, m
    zstep = round(mdt/deltaz)  # Number of cells from RKB up to the bottom

    tvd=tvd[0::deltaz]
    md=md[0::deltaz]

    csgco=500    # Shoe Depth of Conductor Casing, m
    csgc=round(csgco/deltaz)
    csgso=2000   # Shoe Depth of Surface Casing, m
    csgs=round(csgso/deltaz)
    csgio=4000   # Shoe Depth of Intermediate Casing, m
    csgi=round(csgio/deltaz)

    return md,tvd,deltaz,zstep,csgc,csgs,csgi



def heat_coef(rhol,cl,vp,h1,r1,qp,lambdal,r2,h2,rhod,cd,va,r3,h3,qa,lambdar,lambdarw,lambdaw,cr,cw,rhor,rhow,r4,r5,rfm,lambdac,lambdacsr,
              lambdasr,lambdasrfm,cc,csr,rhoc,rhosr,lambdafm,cfm,rhofm,deltaz,deltat,lambdasr2,lambdasr3,lambdacsr2,lambdacsr3,lambdasrfm2,
              lambdasrfm3,csr2,csr3):

    import math

    #HEATCOEFFICIENTS
    # Eq coefficients - Inside Drill String
    c1z = ((rhol * cl * vp) / deltaz) / 2    # Vertical component (North-South) for fluid inside drill string
    c1e = (2 * h1 / r1) / 2   # East component for fluid inside drill string
    c1 = qp / (math.pi * (r1 ** 2))   # Heat source term for fluid inside drill string
    c1t = rhol * cl / deltat    # Time component for fluid inside drill string

    # Eq coefficients - Drill String Wall
    c2z = (lambdal / (deltaz ** 2)) / 2     # Vertical component (North-South) for drill string wall
    c2e = (2 * r2 * h2 / ((r2 ** 2) - (r1 ** 2))) / 2   # East component for drill string wall
    c2w = (2 * r1 * h1 / ((r2 ** 2) - (r1 ** 2))) / 2   # West component for drill string wall
    c2t = rhod * cd / deltat    # Time component for drill string wall

    # Eq coefficients - Inside Annular
    c3z = (rhol * cl * va / deltaz) / 2     # Vertical component (North-South) for fluid inside annular
    c3e = (2 * r3 * h3 / ((r3 ** 2) - (r2 ** 2))) / 2   # East component for fluid inside annular
    c3w = (2 * r2 * h2 / ((r3 ** 2) - (r2 ** 2))) / 2   # West component for fluid inside annular
    c3 = qa / (math.pi * ((r3 ** 2) - (r2 ** 2)))   # Heat source term for fluid inside annular
    c3t = rhol * cl / deltat    # Time component for fluid inside annular

    # Casing
    c4z=[]    # Vertical component (North-South) for casing 
    c4e=[]    # East component for casing
    c4w=[]    # West component for casing
    c4t=[]    # Time component for casing

    # Surrounding Space
    c5z=[]    # Vertical component (North-South) for surrounding space
    c5w=[]    # West component for surrounding space
    c5e=[]    # East component for surrounding space
    c5t=[]    # Time component for surrounding space

    #j < Riser:   (This is the seawater/riser section)
    lambda4=lambdar  #Thermal conductivity of the casing (riser in this section) 
    lambda45=lambdarw   #Comprehensive Thermal conductivity of the casing (riser) and surrounding space (seawater)
    lambda5=lambdaw   #Thermal conductivity of the surrounding space (seawater)
    lambda56=lambdaw   #Comprehensive Thermal conductivity of the surrounding space (seawater) and formation (seawater)
    c4=cr   #Specific Heat Capacity of the casing (riser)
    c5=cw   #Specific Heat Capacity of the surrounding space (seawater)
    rho4=rhor   #Density of the casing (riser)
    rho5=rhow   #Density of the surrounding space (seawater)
    #Casing: (in this section casing=riser)
    c4z1=(lambda4 / (deltaz ** 2)) / 2
    c4e1=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4w1=(2 * r3 * h3 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4t1=rho4 * c4 / deltat
    #Surrounding space: (in this section surrounding space is only seawater )
    c5z1=(lambda5 / (deltaz ** 2)) / 2
    c5w1=(2 * lambda56 / (r5 * (r5 - r4) * math.log(r5 / r4))) / 2
    c5e1=(2 * lambda56 / (r5 * (r5 - r4) * math.log(rfm / r5))) / 2
    c5t1=rho5 * c5 / deltat

    #Riser<=j<csgc:  (This section has intermediate casing + cement + surface casing + cement + conductor casing + cement )
    lambda4 = lambdac   #Thermal conductivity of the casing 
    lambda45=lambdacsr    #Comprehensive Thermal conductivity of the casing and surrounding space
    lambda5=lambdasr    #Thermal conductivity of the surrounding space (seawater)
    lambda56=lambdasrfm   #Comprehensive Thermal conductivity of the surrounding space and formation
    c4 = cc   #Specific Heat Capacity of the casing
    c5 = csr    #Specific Heat Capacity of the surrounding space 
    rho4 = rhoc   #Density of the casing
    rho5 = rhosr    #Density of the surrounding space
    # Casing:
    c4z2=(lambda4 / (deltaz ** 2)) / 2
    c4e2=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4w2=(2 * r3 * h3 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4t2=rho4 * c4 / deltat
    # Surrounding space:  (cement + surface casing + cement + conductor casing + cement )
    c5z2=(lambda5 / (deltaz ** 2)) / 2
    c5w2=(2 * lambda56 / (r5 * (r5 - r4) * math.log(r5 / r4))) / 2
    c5e2=(2 * lambda56 / (r5 * (r5 - r4) * math.log(rfm / r5))) / 2
    c5t2=rho5 * c5 / deltat

    #csgc<=j<csgs:  (This section has intermediate casing + cement + surface casing + cement + formation)
    lambda4 = lambdac   #Thermal conductivity of the casing 
    lambda45 = lambdacsr2    #Comprehensive Thermal conductivity of the casing and surrounding space
    lambda5 = lambdasr2    #Thermal conductivity of the surrounding space
    lambda56 = lambdasrfm2   #Comprehensive Thermal conductivity of the surrounding space and formation
    c4 = cc   #Specific Heat Capacity of the casing
    c5 = csr2    #Specific Heat Capacity of the surrounding space
    rho4 = rhoc   #Density of the casing
    rho5 = rhosr-600    #Density of the surrounding space
    # Casing:
    c4z3=(lambda4 / (deltaz ** 2)) / 2
    c4e3=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4w3=(2 * r3 * h3 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4t3=rho4 * c4 / deltat
    # Surrounding:   (cement + surface casing + cement + formation)
    c5z3=(lambda5 / (deltaz ** 2)) / 2
    c5w3=(2 * lambda56 / (r5 * (r5 - r4) * math.log(r5 / r4))) / 2
    c5e3=(2 * lambda56 / (r5 * (r5 - r4) * math.log(rfm / r5))) / 2
    c5t3=rho5 * c5 / deltat

    #csgs<=j<csgi:   (This section has intermediate casing + cement + formation)
    lambda4 = lambdac   #Thermal conductivity of the casing 
    lambda45 = lambdacsr3    #Comprehensive Thermal conductivity of the casing and surrounding space
    lambda5 = lambdasr3    #Thermal conductivity of the surrounding space
    lambda56 = lambdasrfm3   #Comprehensive Thermal conductivity of the surrounding space and formation
    c4 = cc   #Specific Heat Capacity of the casing
    c5 = csr3    #Specific Heat Capacity of the surrounding space
    rho4 = rhoc   #Density of the casing
    rho5 = rhosr-1200   #Density of the surrounding space
    # Casing:
    c4z4=(lambda4 / (deltaz ** 2)) / 2
    c4e4=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4w4=(2 * r3 * h3 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4t4=rho4 * c4 / deltat
    # Surrounding space:   (cement + formation)
    c5z4=(lambda5 / (deltaz ** 2)) / 2
    c5w4=(2 * lambda56 / (r5 * (r5 - r4) * math.log(r5 / r4))) / 2
    c5e4=(2 * lambda56 / (r5 * (r5 - r4) * math.log(rfm / r5))) / 2
    c5t4=rho5 * c5 / deltat

    #j >= csgi:    (This section is open hole)
    lambda4 = lambdafm    #Thermal conductivity of the casing (formation in this section) 
    lambda45 = lambdafm   #Comprehensive Thermal conductivity of the casing (formation) and surrounding space (formation)
    lambda5 = lambdafm    #Thermal conductivity of the surrounding space (formation)
    lambda56 = lambdafm   #Comprehensive Thermal conductivity of the surrounding space and formation
    c4 = cfm    #Specific Heat Capacity of the casing (formation)
    c5 = cfm    #Specific Heat Capacity of the surrounding space (formation)
    rho4 = rhofm    #Density of the casing (formation)
    rho5 = rhofm    #Density of the surrounding space (formation)
    # Casing:
    c4z5=(lambda4 / (deltaz ** 2)) / 2
    c4e5=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4w5=(2 * lambda45 / ((r4 ** 2) - (r3 ** 2))) / 2
    c4t5=rho4 * c4 / deltat
    # Surrounding space:     (only formation)
    c5z5=(lambda5 / (deltaz ** 2)) / 2
    c5w5=(2 * lambda56 / (r5 * (r5 - r4) * math.log(r5 / r4))) / 2
    c5e5=(2 * lambda56 / (r5 * (r5 - r4) * math.log(rfm / r5))) / 2
    c5t5=rho5 * c5 / deltat

    return c1z,c1e,c1,c1t,c2z,c2e,c2w,c2t,c3z,c3e,c3w,c3,c3t,c4z,c4e,c4w,c4t,c5z,c5w,c5e,c5t,c4z1,c4e1, \
            c4w1,c4t1,c5z1,c5w1,c5e1,c5t1,c4z2,c4e2,c4w2,c4t2,c5z2,c5w2,c5e2,c5t2,c4z3,c4e3,c4w3,c4t3,c5z3, \
            c5w3,c5e3,c5t3,c4z4,c4e4,c4w4,c4t4,c5z4,c5w4,c5e4,c5t4,c4z5,c4e5,c4w5,c4t5,c5z5,c5w5,c5e5,c5t5



import matplotlib.pyplot as plt

def plot_temp_time(finaltime,Tbot,Tout,Tfm):

    # Plotting Tbottom and Tout through time
    plt.plot(range(finaltime), Tbot, 'b', label='Bottom')  # Temp. inside Annulus vs Time
    plt.plot(range(finaltime), Tout, 'r', label='Outlet (Annular)')  # Temp. inside Annulus vs Time
    plt.axhline(y=Tfm[-1], color='k', label='Formation')  # Temp. inside Formation vs Time
    plt.xlim(0, finaltime - 1)
    plt.xlabel('Time, h')
    plt.ylabel('Temperature, °C')
    plt.legend()  # applying the legend
    plt.show()


def plot_temp_profile(Tdsi,Ta,Tr,Tcsg,Tfm,Tsr,Riser,md):

    # Plotting Temperature PROFILE
    plt.plot(Tdsi, md, c='r', label='Fluid in Drill String')  # Temp. inside Drillpipe vs Depth
    plt.plot(Ta, md, 'b', label='Fluid in Annulus')
    if Riser > 0:
        plt.plot(Tr, md, 'g', label='Temp. - Riser')  # Temp. due to gradient vs Depth
    plt.plot(Tcsg, md, 'c', label='Casing')  # Temp. due to gradient vs Depth
    plt.plot(Tfm, md, 'g', label='Formation')  # Temp. due to gradient vs Depth
    plt.plot(Tsr, md, c='k', ls='-', marker='', label='Surrounding Space')  # Temp. due to gradient vs Depth
    plt.xlabel('Temperature, °C')
    plt.ylabel('Depth, m')
    title = 'Temperature profile stabilized ' #at %i hours' % finaltime
    plt.title(title)
    plt.ylim(plt.ylim()[::-1])  # reversing y axis
    plt.legend()  # applying the legend
    plt.show()


from math import pi

class WellTemperature(object):
    def __init__(self):
        self.tin = 30  # Inlet Fluid temperature, °C
        self.ts = 15  # Surface Temperature (RKB), °C
        self.xi = 5  # number of radial sections (Inside DS, DS wall, Annular, Csg and Surrounding Space)
        self.wd = 100  # Water Depth, m
        deltaz=50  # deltaz is always the same  # Length of each grid cell
        self.riser = round(self.wd / deltaz)  # number of grid cells for the riser
        # Wellbore Geometry
        self.ddi = 0.101  # Drill String Inner  Diameter, m
        self.ddo = 0.114  # Drill String Outer Diameter, m
        self.dcsg = 0.216  # Casing Inner Diameter, m
        self.dcsgo = 0.24  # Casing Outer Diameter, m
        self.dsr = 0.26  # Surrounding Space Inner Diameter, m
        self.dsro = 0.6  # Surrounding Space Outer Diameter, m
        self.dri = 0.45  # Riser diameter Inner Diameter, m
        self.dro = 0.5334  # Riser diameter Outer Diameter, m
        self.dfm = 2  # Undisturbed Formation Diameter, m
        self.dcsgci = 0.63   # Conductor casing inner diameter, m
        self.dcsgco = 0.66   # Conductor casing outer diameter, m
        self.dcemo = self.dcsgco+0.03  # First Cement sheath outer diameter, m
        self.dcsgsi = 0.41   # Surface casing inner diameter, m
        self.dcsgso = 0.44   # Surface casing outer diameter, m
        self.r1 = self.ddi / 2  # Drill String Inner  Radius, m
        self.r2 = self.ddo / 2  # Drill String Outer Radius, m
        self.r3 = self.dcsg / 2  # Casing Inner Radius, m
        self.r3r = self.dri / 2  # Riser Inner Radius, m
        self.r4r = self.dro / 2  # Riser Outer Radius, m
        self.r4 = self.dcsgo / 2  # Surrounding Space Inner Radius m
        self.r5 = self.dcemo / 2  # Surrounding Space Outer Radius, m
        self.rfm = self.dfm / 2  # Undisturbed Formation Radius, m
        self.tcsr = (self.dcsgco-self.dcsgci)/2 + (self.dcsgso-self.dcsgsi)/2
        self.tcem = (self.dcsgsi-self.dcsgo)/2 + (self.dcsgci-self.dcsgso)/2 + (self.dcemo-self.dcsgco)/2
        self.tcsr2 = (self.dcsgso-self.dcsgsi)/2
        self.tcem2 = (self.dcsgsi-self.dcsgo)/2 + (self.dcsgci-self.dcsgso)/2
        self.tcsr3 = 0
        self.tcem3 = (self.dcsgsi - self.dcsgo) / 2
        self.tcem4 = 0
    # Flow Rate
        self.q = 47.696     # Flow rate, m^3/h
        self.va = (self.q / (pi * ((self.r3 ** 2) - (self.r2 ** 2)))) / 3600        # Fluid velocity through the annular
        self.vp = (self.q / (pi * (self.r1 ** 2))) / 3600       # Fluid velocity through the drill pipe
    # Heat Coefficients
    # Thermal Conductivity, W/(m*°C)
        self.lambdal = 0.635     # Fluid
        self.lambdac = 43.3    # Casing
        self.lambdacem = 0.7     #Cement
        self.lambdad = 40     # Drill Pipe
        self.lambdasr = (self.lambdac*(self.tcsr) + self.lambdacem*(self.tcem))/(self.r5-self.r4)       # Surrounding space
        self.lambdasr2 = (self.lambdac*(self.tcsr2) + self.lambdacem*(self.tcem2))/(self.r5 - self.r4)     # Surrounding space
        self.lambdasr3 = (self.lambdac*(self.tcsr3) + self.lambdacem*(self.tcem3))/(self.r5 - self.r4)  # Surrounding space
        self.lambdacsr = (self.lambdac * (self.r4 - self.r3) + self.lambdasr * (self.r5 - self.r4)) / (
                    self.r5 - self.r3)  # Comprehensive Casing - Surrounding space
        self.lambdacsr2 = (self.lambdac * (self.r4 - self.r3) + self.lambdasr2 * (self.r5 - self.r4)) / (
                    self.r5 - self.r3)  # Comprehensive Casing - Surrounding space
        self.lambdacsr3 = (self.lambdac * (self.r4 - self.r3) + self.lambdasr3 * (self.r5 - self.r4)) / (
                    self.r5 - self.r3)  # Comprehensive Casing - Surrounding space
        self.lambdafm = 2.249       # Formation
        self.lambdasrfm = (self.lambdac * (self.r5 - self.r4) + self.lambdasr * (self.rfm - self.r5)) / (
                    self.rfm - self.r4)  # Comprehensive Surrounding space - Formation
        self.lambdasrfm2 = (self.lambdac * (self.r5 - self.r4) + self.lambdasr2 * (self.rfm - self.r5)) / (
                    self.rfm - self.r4)  # Comprehensive Surrounding space - Formation
        self.lambdasrfm3 = (self.lambdac * (self.r5 - self.r4) + self.lambdasr3 * (self.rfm - self.r5)) / (
                    self.rfm - self.r4)  # Comprehensive Surrounding space - Formation
        self.lambdar = 15.49     # Riser
        self.lambdarw = 5       # Comprehensive Riser - Seawater
        self.lambdaw = 0.6     # Seawater
    # Specific Heat Capacity, J/(kg*°C)
        self.cl = 3713       # Fluid
        self.cc = 469    # Casing
        self.ccem = 2000     # Cement
        self.cd = 400     # Drill Pipe
        self.cr = 464     # Riser
        self.cw = 4000       # Seawater
        self.csr = (self.cc*(self.tcsr) + self.ccem*(self.tcem))/(self.r5-self.r4)       # Surrounding space
        self.csr2 = (self.cc * (self.tcsr2) + self.ccem * (self.tcem2)) / (self.r5 - self.r4)  # Surrounding space
        self.csr3 = (self.cc * (self.tcsr3) + self.ccem * (self.tcem3)) / (self.r5 - self.r4)  # Surrounding space
        self.cfm = 1500       # Formation
    # Convective Heat Transfer Coefficient, W/(m^2*°C)
        self.h1 = 2000       # Drill Pipe inner wall
        self.h2 = 2000       # Drill Pipe outer wall
        self.h3 = 200       # Casing inner wall
        self.h3r = 200    # Riser inner wall
    # Densities, kg/m3
        self.rhol = 1198       # Fluid
        self.rhod = 8000       # Drill Pipe
        self.rhoc = 9000       # Casing
        self.rhor = 9000       # Riser
        self.rhofm = 2645     # Formation
        self.rhow = 1029       # Seawater
        self.rhosr = 4000     # Surrounding Space
    # Thermal Gradients
        self.gt = 0.0238 * deltaz  # Geothermal gradient, °C/m
        self.wtg = -0.005 * deltaz  # Seawater thermal gradient, °C/m
    # Operational Parameters    
        self.rpm = 100    # Revolutions per minute
        self.t = 2     # Torque on the drill string, kN*m
        self.tbit = 1.35       # Torque on the bit, kN*m
        self.wob = 22.41     # Weight on bit, kN
        self.rop = 14.4     # Rate of Penetration, m/h
        self.an = 2       # Area of the nozzles, m^2
        self.mdt = 5000     # Measured Depth of the Target, m
    # Heat Source Terms
        self.qp = 2*pi * (self.rpm/60) * self.t * 2 * 0.24 * self.rhol * (self.vp ** 2) * (self.mdt / (self.ddi*127.094*10**6)) * (1/0.24**.5)
        self.qa = 0.05*(self.wob*(self.rop/3600)+2*pi*(self.rpm/60)*self.tbit) + (self.rhol/2*9.81)*((self.q/3600)/(0.095*self.an)) \
                + (2*0.3832*self.mdt/((self.r3-self.r2)*(127.094*10**6)))*((2*(0.7+1)*self.va)/(0.7*pi*(self.r3+self.r2)
                * (self.r3-self.r2)**2))**0.7




def temp_calc(Tin,Tdsio, Tdso, Tao, Tcsgo, Tsro,c1z,c1e,c1,c1t,c2z,c2e,c2w,c2t,c3z,c3e,c3w,c3,c3t,c4z,c4e,c4w,c4t,
            c5z,c5w,c5e,c5t,c4z1,c4e1,c4w1,c4t1,c5z1,c5w1,c5e1,c5t1,c4z2,c4e2,c4w2,c4t2,c5z2,c5w2,c5e2,c5t2,c4z3,
            c4e3,c4w3,c4t3,c5z3,c5w3,c5e3,c5t3,c4z4,c4e4,c4w4,c4t4,c5z4,c5w4,c5e4,c5t4,c4z5,c4e5,c4w5,c4t5,c5z5,
            c5w5,c5e5,c5t5,zstep,Riser,xi,csgc,csgs,csgi):

    import numpy as np

    Tdsi = [Tin]
    Tds = []
    Ta = []
    Tcsg = []
    Tsr = []

    # Creating vectors N,W,C,E,S,B
    N = []
    W = []
    C = []
    E = []
    S = []
    B = []

    # Casing:
    c4z = c4z1
    c4e = c4e1
    c4w = c4w1
    c4t = c4t1
    # Surrounding:
    c5z = c5z1
    c5w = c5w1
    c5e = c5e1
    c5t = c5t1

    for j in range(zstep):

        if j==Riser:
            # Casing:
            c4z = c4z2
            c4e = c4e2
            c4w = c4w2
            c4t = c4t2
            # Surrounding:
            c5z = c5z2
            c5w = c5w2
            c5e = c5e2
            c5t = c5t2

        if j==csgc:
            # Casing:
            c4z = c4z3
            c4e = c4e3
            c4w = c4w3
            c4t = c4t3
            # Surrounding:
            c5z = c5z3
            c5w = c5w3
            c5e = c5e3
            c5t = c5t3

        if j==csgs:
            # Casing:
            c4z = c4z4
            c4e = c4e4
            c4w = c4w4
            c4t = c4t4
            # Surrounding:
            c5z = c5z4
            c5w = c5w4
            c5e = c5e4
            c5t = c5t4

        if j==csgi:
            # Casing:
            c4z = c4z5
            c4e = c4e5
            c4w = c4w5
            c4t = c4t5
            # Surrounding:
            c5z = c5z5
            c5w = c5w5
            c5e = c5e5
            c5t = c5t5

        for i in range(xi):
            if i == 0:  # Inside Drill String
                if j == 1:
                    W.append(0)
                    C.append(c1t + c1e + c1z)
                    E.append(-c1e)
                    S.append(0)
                    B.append(
                        c1t * Tdsio[j] + c1 + c1e * (Tdso[j] - Tdsio[j]) + c1z * (Tdsio[j - 1] - Tdsio[j]) + c1z * (Tdsi[j - 1]))

                if j > 1 and j < zstep - 1:
                    N.append(-c1z)
                    W.append(0)
                    C.append(c1t + c1e + c1z)
                    E.append(-c1e)
                    S.append(0)
                    B.append(c1t * Tdsio[j] + c1 + c1e * (Tdso[j] - Tdsio[j]) + c1z * (Tdsio[j - 1] - Tdsio[j]))

                if j == zstep - 1:
                    N.append(-c1z - c2z)
                    W.append(0)
                    C.append(3 * c1t + c1z + c2z + c3e)
                    E.append(-c3e)
                    B.append(c1t * Tdsio[j] + c1 + c1z * (Tdsio[j - 1] - Tdsio[j]) + c1t * Tdso[j] + c1t * Tao[j])

            if i == 1:  # Drill string wall

                if j == 0:
                    C.append(c2t + c2e + c2w + c2z)
                    E.append(-c2e)
                    S.append(-c2z)
                    B.append(c2t * Tdso[j] + c2w * Tdsi[j] + c2e * (Tao[j] - Tdso[j]) + c2w * (Tdsio[j] - Tdso[j]) + c2z * (
                            Tdso[j + 1] - Tdso[j]))

                if j > 0 and j < zstep - 1:
                    N.append(-c2z)
                    W.append(-c2w)
                    C.append(c2t + c2e + c2w + 2 * c2z)
                    E.append(-c2e)
                    if j < zstep - 2:
                        S.append(-c2z)
                    B.append(c2t * Tdso[j] + c2e * (Tao[j] - Tdso[j]) + c2w * (Tdsio[j] - Tdso[j]) + c2z * (
                            Tdso[j + 1] - Tdso[j]) + c2z * (Tdso[j - 1] - Tdso[j]))

            if i == 2:  # Annular

                if j == 0:
                    W.append(-c3w)
                    C.append(c3t + c3e + c3w + c3z)
                    E.append(-c3e)
                    S.append(-c3z)
                    B.append(
                        c3t * Tao[j] + c3 + c3e * (Tcsgo[j] - Tao[j]) + c3w * (Tdso[j] - Tao[j]) + c3z * (Tao[j + 1] - Tao[j]))

                if j > 0 and j < zstep - 1:
                    N.append(0)
                    W.append(-c3w)
                    C.append(c3t + c3e + c3w + c3z)
                    E.append(-c3e)
                    if j < zstep - 2:
                        S.append(-c3z)
                    B.append(
                        c3t * Tao[j] + c3 + c3e * (Tcsgo[j] - Tao[j]) + c3w * (Tdso[j] - Tao[j]) + c3z * (Tao[j + 1] - Tao[j]))

            if i == 3:  # Casing

                if j == 0:
                    W.append(-c4w)
                    C.append(c4t + c4e + c4w + c4z)
                    E.append(0)
                    S.append(-c4z)
                    B.append(
                        c4t * Tcsgo[j] + c4e * Tsro[j] + c4e * (Tsro[j] - Tcsgo[j]) + c4w * (Tao[j] - Tcsgo[j]) + c4z * (
                                Tcsgo[j + 1] - Tcsgo[j]))

                if j > 0 and j < zstep - 1:
                    N.append(-c4z)
                    W.append(-c4w)
                    C.append(c4t + c4e + c4w + 2 * c4z)
                    E.append(0)
                    S.append(-c4z)
                    B.append(
                        c4t * Tcsgo[j] + c4e * Tsro[j] + c4e * (Tsro[j] - Tcsgo[j]) + c4w * (Tao[j] - Tcsgo[j]) + c4z * (
                                Tcsgo[j + 1] - Tcsgo[j])
                        + c4z * (Tcsgo[j - 1] - Tcsgo[j]))

                if j == zstep - 1:
                    N.append(-c4z)
                    W.append(-c4w)
                    C.append(c4t + c4e + c4w + c4z)
                    E.append(0)
                    B.append(
                        c4t * Tcsgo[j] + c4e * Tsro[j] + c4e * (Tsro[j] - Tcsgo[j]) + c4w * (Tao[j] - Tcsgo[j]) + c4z * (
                                Tcsgo[j - 1] - Tcsgo[j]))

            if i == 4:  # Formation

                if j == 0:
                    W.append(-c5w)
                    C.append(c5w + c5z + c5e + c5t)
                    E.append(0)
                    S.append(-c5z)
                    B.append(c5w * (Tcsgo[j] - Tsro[j]) + c5z * (Tsro[j + 1] - Tsro[j]) + c5e * Tsro[j] + c5t * Tsro[j])

                if j > 0 and j < zstep - 1:
                    N.append(-c5z)
                    W.append(-c5w)
                    C.append(c5w + c5e + 2 * c5z + c5t)
                    E.append(0)
                    S.append(-c5z)
                    B.append(
                        c5w * (Tcsgo[j] - Tsro[j]) + c5z * (Tsro[j + 1] - Tsro[j]) + c5z * (Tsro[j - 1] - Tsro[j]) + c5e *
                        Tsro[j] + c5t * Tsro[j])

                if j == zstep - 1:
                    N.append(-c5z)
                    W.append(-c5w)
                    C.append(c5w + c5e + c5z + c5t)
                    B.append(c5w * (Tcsgo[j] - Tsro[j]) + c5z * (Tsro[j - 1] - Tsro[j]) + c5e * Tsro[j] + c5t * Tsro[j])

    #LINEARSYSTEM
    # Creating pentadiagonal matrix
    A = np.zeros((xi * zstep - 3, xi * zstep - 3))

    # Filling up Pentadiagonal Matrix A
    lenC = xi * zstep - 3
    lenN = lenC - xi
    lenW = lenC - 1
    lenE = lenC - 1
    lenS = lenC - xi

    for it in range(lenC):  # Inserting list C
        A[it, it] = C[it]
    for it in range(lenE):  # Inserting list E
        A[it, it + 1] = E[it]
    for it in range(lenW):  # Inserting list W
        A[it + 1, it] = W[it]
    for it in range(lenN):  # Inserting list N
        A[it + xi, it] = N[it]
    for it in range(lenS):  # Inserting list S
        A[it, it + xi] = S[it]

    A[lenC - 1 - (xi - 3) - (xi - 1), lenC - 1 - (xi - 3)] = -c2z
    A[lenC - 1 - (xi - 3) - (xi - 2), lenC - 1 - (xi - 3)] = -c3z

    Temp = np.linalg.solve(A, B)

    for x in range(zstep):
        if x < zstep - 1:
            Tds.append(Temp[5 * x])
        if x == zstep - 1:
            Tds.append(Temp[lenC - 1 - (xi - 3)])
    for x in range(zstep - 1):
        if x < zstep - 2:
            Tdsi.append(Temp[5 * x + 4])
        if x == zstep - 2:
            Tdsi.append(Temp[lenC - 1 - (xi - 3)])
    for x in range(zstep):
        if x < zstep - 1:
            Ta.append(Temp[5 * x + 1])
        if x == zstep - 1:
            Ta.append(Temp[lenC - 1 - (xi - 3)])
    for x in range(zstep):
        if x < zstep - 1:
            Tcsg.append(Temp[5 * x + 2])
        if x == zstep - 1:
            Tcsg.append(Temp[lenC - 2])
    for x in range(zstep):
        if x < zstep - 1:
            Tsr.append(Temp[5 * x + 3])
        if x == zstep - 1:
            Tsr.append(Temp[lenC - 1])

    Tr=Tcsg[:Riser]+[None]*(zstep-Riser)
    for x in range(Riser):
        Tcsg[x]=None

    return Tdsi,Ta,Tr,Tcsg,Tsr